/*
 * Adobe MAX 2010 PhotoReadr Demo Application - andyMatthews.net
 */
$(document).ready(function(){

	/* VARIABLE DECLARATIONS */
	// used for ajax call, and for network detection
	var domain = 'http://www.andymatthews.net';
	// data retrieval service
	var requestURL = domain + '/code/max2010/flickr.cfc?returnFormat=json&method=getImages';
	// file object pointing to the db file
	var dbFile = air.File.applicationStorageDirectory.resolvePath('max2010.sqlite');
	// image data pulled from server
	var jsonData = '';

	// UI VARIABLES
	var $download = $('#download');
	var $manage = $('#manage');
	var $downloadPhotos = $('#downloadPhotos');
	var $managePhotos = $('#managePhotos');





	/* INTERFACE BINDINGS */
	// CONNECTION RESULT EVENT LISTENER
	$(document).bind('connectionResult', function(event, status){
		var position = (status) ? '0px -56px' : '0px -28px';
		$('#status').css('background-position',position);
		if (status) $download.trigger('click');
	});

	// GET REMOTE DATA
	$download.bind('click', function(){
		// change view state
		displayPanels($downloadPhotos,$managePhotos);
		// display the loading image
		toggleLoading(true);
		// retrieve remote data
		$.get(
			requestURL,
			function(data,status,request){
				// parse out the JSON data
				jsonData = JSON.parse(data);
				// tmp html string
				var html = '';
				var imgPreviewTmpl = '<div><a href="LARGE"><img src="THUMB"/></a><br />' +
												'<input type="checkbox" name="images" id="iIMGID" value="IMGID" /><label for="iIMGID">Save Image</label></div>';
				// loop over the data packet
				$.each(jsonData, function(i, item) {
					// find and replace, using the template, add it to the rest of the HTML string
					html += imgPreviewTmpl.replace(/LARGE/gi,item.large).replace(/THUMB/gi,item.thumb).replace(/IMGID/gi,i);
				});
				// tack a submit button on to the end of the html string
				html += '<input type="submit" value="Save Selected Images" />';
				// insert the newly created string of html
				$downloadPhotos.html(html);
				// hide the loading image
				toggleLoading(false);
			}
		);
	});

	// OPENING IMAGES IN A NEW WINDOW
	$('#downloadPhotos div a').live('click', function(e){
		e.preventDefault();
		// get the URL to be opened
		var href = $(this).attr('href');
		// set options for the window
		var options = new air.NativeWindowInitOptions(); 
		var windowBounds = new air.Rectangle(300,300,300,300); 
		newHTMLLoader = air.HTMLLoader.createRootWindow(true, options, true, windowBounds); 
		newHTMLLoader.load(new air.URLRequest(href));
	});

	// DOWNLOAD IMAGES AND INSERT RECORDS
	$downloadPhotos.bind('submit', function(e){
		e.preventDefault();
		// get indexes of the selected photos
		var numArr = $(this).formSerialize().match(/[0-9]+/gi);
		// make sure that numArr isn't empty
		if (numArr == null) {
			alert('Please select at least one image to download');
		} else {
			// loop over the array to get the URLs of the larger version
			for (var p=0; p<numArr.length; p++) {
				var photo = jsonData[numArr[p]];
				var remoteURL = photo['large'];
				// where do we want to put downloaded images?
				var localURL = air.File.applicationStorageDirectory.resolvePath( remoteURL.match(/[a-z0-9\._]+$/i) );
				// trigger the file download
				downloadFile(remoteURL, localURL);
				// and insert a record into the data for use in tagging
				insertPhotoRecord(photo);
			}
			// trigger the download event again to get new images
			$download.trigger('click');
		}
	});

	// MANAGE LOCAL IMAGES
	$manage.bind('click', function(){
		// toggle window to full screen to make editing easier
		nativeWindow.stage.displayState = runtime.flash.display.StageDisplayState.FULL_SCREEN_INTERACTIVE;
		// change view state
		displayPanels($managePhotos,$downloadPhotos);
		// show the loading image
		toggleLoading(true);
		// query to get the saved pictures
		var picQry = selectPhotos();
		// tmp html string
		var html = '';
		var imgManageTmpl = '<div><a href="PHOTOURL" target="_blank"><img src="app-storage:/FILENAME"/></a>' +
										'<form><input type="hidden" name="imageid" id="imageid" value="ID"><label>Author</label>' +
										'<span>AUTHOR</span><label>Tags</label><span class="tags">TAGS</span><label>Add New Tags<br />' + 
										'<i>Separate new tags with a comma</i></label><textarea name="tags" id="tags"></textarea>' +
										'<input type="submit" value="Add tags"></form></div>';
		// loop over the data packet
		$.each(picQry, function(i, item) {
			// get tags for this image
			var tags = item['tags'] || '';
			// find and replace, using the template, add it to the rest of the HTML string
			html += imgManageTmpl.replace(/PHOTOURL/g,item['photoURL']).replace(/FILENAME/g,item['filename']).replace(/ID/g,item['id']).replace(/AUTHOR/g,item['author']).replace(/TAGS/g,tags);
		});
		// update the HTML
		updateHTML($managePhotos, html);
		// hide the loading image
		toggleLoading(false);
	});

	// CAPTURE THE "ADD TAGS" SUBMIT EVENT
	$('div form', $managePhotos).live('submit', function(e){
		// stop the standard submit
		e.preventDefault();
		// reference the current form
		var $form = $(this);
		// get the value of the textarea field
		var tagsArr = e.currentTarget[1].value.split(',');
		// placeholder for when we add tags to the visual display
		var strTags = '';
		// make sure there's tags
		if (tagsArr.length) {
			// get the image id
			var imageID = parseInt(e.currentTarget[0].value);
			// loop over the tags array and insert them
			$.each(tagsArr, function(t, tag){
				// insert into the database
				insertTag(tag, imageID);
				strTags += ',' + tag;
			});
			// update the HTML
			updateHTML($('.tags', $form), strTags);
			// clear all selected fields
			$form.clearForm();
		}
	});





	/* SQL FUNCTIONS */
	// CHECK FOR DB
	function checkForDB() {
		// ship the app with an empty database rather than a series of create table scripts.
		// if there's not a database file in the applicationDirectory, copy it there
		if (!dbFile.exists) {
			var tmpDB = air.File.applicationDirectory.resolvePath('max2010.sqlite');
			tmpDB.copyTo(dbFile, true);
		}
	}

	// INSERT INITIAL PHOTO RECORD
	function insertPhotoRecord(photoArr) {
		var db = new air.SQLConnection();
		db.open(dbFile);
		dbQuery = new air.SQLStatement(); 
		dbQuery.sqlConnection = db; 
		dbQuery.text = 'INSERT INTO pictures (author,photoURL,filename) VALUES (@author,@photoURL,@filename)';
		dbQuery.parameters['@author'] = photoArr['author'];
		dbQuery.parameters['@photoURL'] = photoArr['photoURL'];
		dbQuery.parameters['@filename'] = photoArr['large'].match(/[a-z0-9\._]+$/i);
		dbQuery.execute();
	}

	// SELECT PHOTOS FROM DATABASE
	function selectPhotos() {
		var db = new air.SQLConnection();
		db.open(dbFile);
		dbQuery = new air.SQLStatement(); 
		dbQuery.sqlConnection = db; 
		dbQuery.text = 'SELECT p.id, p.filename, p.author, p.photoURL, GROUP_CONCAT(t.tag) as tags FROM pictures p LEFT OUTER JOIN tags t ON t.fkPicture = p.id GROUP BY p.id, p.filename, p.author ORDER BY p.id DESC';
		dbQuery.execute();
		return dbQuery.getResult().data;
	}

	// INSERT CUSTOM TAG FOR IMAGE
	function insertTag(tag, fkPicture) {
		var db = new air.SQLConnection();
		db.open(dbFile);
		dbQuery = new air.SQLStatement(); 
		dbQuery.sqlConnection = db; 
		dbQuery.text = 'INSERT INTO tags (tag, fkPicture) VALUES (@tag,@fkPicture)';
		dbQuery.parameters['@tag'] = tag;
		dbQuery.parameters['@fkPicture'] = fkPicture;
		dbQuery.execute();
	}





	/* HELPER FUNCTIONS */
	// CHECK FOR CONNECTION
	function checkConnection() {
		// create new monitor method connection
		var monitor = new air.URLMonitor(new air.URLRequest(domain));
		// listen for the status event 
		monitor.addEventListener(air.StatusEvent.STATUS, function(evt){
			$(document).trigger('connectionResult',evt.currentTarget.available);
		});
		// start monitoring 
		monitor.start();
	}

	// TOGGLE LOADING IMAGE
	function toggleLoading(state) {
		var display = (state) ? 'block' : 'none';
		$('#loading').css('display', display);
	}

	// UPDATE HTML
	function updateHTML($obj, htmlStr) {
		$obj.html(function(i, current) {
			return (current += htmlStr).replace(/^,/gi,'').replace(/,/g,', ');
		}); 
	}

	// DOWNLOAD FILE
	function downloadFile(url, fileName) {
		var req = new air.URLRequest(url);
		var stream = new air.URLStream();
		stream.addEventListener(air.Event.COMPLETE, writeAirFile);
		stream.load(req);
	
		function writeAirFile(e) {
			//air.Introspector.Console.log(e);	
			var fileData = new air.ByteArray();
			stream.readBytes(fileData,0,stream.bytesAvailable);
			var fileStream = new air.FileStream();
			fileStream.openAsync(fileName, air.FileMode.WRITE);
			fileStream.writeBytes(fileData,0);
			fileStream.close();	
		}
	}

	// DISPLAY PANELS
	function displayPanels($active,$inactive) {
		// setup panels for display
		$inactive.hide();
		$active.html('').show();
	} 

	// SET UP ICON TRAY
	function setupIconTray() {
		// shortcut to the nativeApplication object
		var app = air.NativeApplication.nativeApplication;
		app.addEventListener(air.Event.COMPLETE, iconLoadComplete);
		// create new instance of icon loader
		var iconLoader = new air.Loader();
		if(air.NativeApplication.supportsSystemTrayIcon){
			iconLoader.contentLoaderInfo.addEventListener(air.Event.COMPLETE, iconLoadComplete);
			iconLoader.load(new air.URLRequest("icons/AIRApp_16.png"));
			app.icon.addEventListener(window.runtime.flash.events.MouseEvent.CLICK, restore);
			app.icon.tooltip = "PhotoReadr";
		}
 		function iconLoadComplete(event) {
			app.icon.bitmaps = new runtime.Array(event.target.content.bitmapData);
		}
		// this restores the current window to the visible desktop area
		function restore() {
			nativeWindow.visible = true;
			nativeWindow.restore();
			nativeWindow.activate();
			nativeWindow.orderToFront();
		}
	}
 
 	// FIRE OFF APPLICATION
	function init() {
		// announce internet connection status
		checkConnection();
		// copy the database to the storage directory if it doesn't exist
		checkForDB();
		// add icon to the tray
		setupIconTray();
	}

	// TRIGGER THE APP
	init();

});